'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var MagicString = require('magic-string');
var shiki = require('shiki');
var compiler = require('svelte/compiler');
var promises = require('fs/promises');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var MagicString__default = /*#__PURE__*/_interopDefaultLegacy(MagicString);

const defaultMapExtension = {
  svg: "xml"
};
const SYNTAX_SUFFIX = "?syntax";
const SYNTAX_EXTENSION = ".syntax";
const EXTENSION_REGEXP = /([^\.]+)(\.syntax)?$/;
function wrapPath(path) {
  return "\0" + path + SYNTAX_EXTENSION;
}
function unWrapPath(path) {
  return path.slice(1, path.lastIndexOf(SYNTAX_EXTENSION));
}
const basePlugin = (handleContent) => {
  return function(options) {
    const { mapExtension, ...shikiOptions } = {
      mapExtension: {
        ...defaultMapExtension,
        ...options?.mapExtension
      },
      ...options
    };
    return {
      name: "syntax-highlight",
      enforce: "pre",
      async resolveId(id, importer, options2) {
        if (!id.endsWith(SYNTAX_SUFFIX)) {
          return null;
        }
        const suffixIndex = id.indexOf(SYNTAX_SUFFIX);
        const path = id.slice(0, suffixIndex);
        const resolution = await this.resolve(path, importer, {
          skipSelf: true,
          ...options2
        });
        if (!resolution) {
          this.warn(`"${id}" couldn't be resolved, falling back to default`);
          return null;
        }
        return wrapPath(resolution.id);
      },
      async load(id, loadOptions) {
        if (!id.endsWith(SYNTAX_EXTENSION)) {
          return null;
        }
        const path = id.startsWith("\0") ? unWrapPath(id) : id;
        let content;
        try {
          content = await promises.readFile(path, "utf-8");
        } catch (ex) {
          this.warn(`"${id}" couldn't be loaded, falling back to default`);
          return;
        }
        const extensionMatch = path.match(EXTENSION_REGEXP);
        if (!extensionMatch) {
          this.warn(
            `"${id}" doesn't have an extension, falling back to default`
          );
          return;
        }
        const extension = extensionMatch[1];
        const language = mapExtension[extension] || extension;
        return handleContent({
          content,
          shikiOptions,
          language,
          loadOptions
        });
      }
    };
  };
};

const paramRegExp = /\$\[(\w+)(?::(\w+))?\]/g;
const syntaxHighlight = ({
  ssr,
  ...baseOptions
} = {}) => {
  const plugin = basePlugin(
    async ({ content, shikiOptions, language, loadOptions }) => {
      const highlighter = await shiki.getHighlighter(shikiOptions);
      const { replacements, transformedCode, params } = transformAndRetriveReplacements(content, paramRegExp);
      const lines = highlighter.codeToThemedTokens(
        transformedCode,
        language,
        void 0,
        {
          includeExplanation: false
        }
      );
      const { fg, bg } = highlighter.getTheme();
      const s = new MagicString__default["default"](content);
      s.prepend(`<pre style="background-color: ${bg}"><code>`);
      let replacement = replacements.shift();
      let index = 0;
      for (const line of lines) {
        s.appendLeft(index, `<span class="line">`);
        for (const token of line) {
          let styles = `color: ${token.color || fg}`;
          if (token.fontStyle !== void 0) {
            if (token.fontStyle & 1 /* Italic */) {
              styles += "; font-style: italic";
            }
            if (token.fontStyle & 2 /* Bold */) {
              styles += "; font-weight: bold";
            }
            if (token.fontStyle & 4 /* Underline */) {
              styles += "; text-decoration: underline";
            }
          }
          const tokenEnd = index + token.content.length;
          let offset = 0;
          let currentStart = 0;
          let newContent = "";
          while (replacement && replacement.start < tokenEnd + offset) {
            const { start, param, value, length } = replacement;
            const beforeParam = token.content.slice(
              currentStart,
              start - index - offset
            );
            newContent += escapeHtml(beforeParam) + `{#if $$slots["${param}"]}<slot name="${param}" />{:else}{${param}}{/if}`;
            currentStart += beforeParam.length + value.length;
            offset += length - value.length;
            replacement = replacements.shift();
          }
          newContent += escapeHtml(token.content.slice(currentStart));
          s.overwrite(
            index,
            tokenEnd + offset,
            `<span class="token" style="${styles}">${newContent}</span>`
          );
          index = tokenEnd + offset;
        }
        s.appendRight(index, `</span>`);
        index++;
      }
      s.append(`</code></pre>`);
      if (params.length) {
        s.prepend(`
<script lang="ts">
${params.map(({ param, value }) => {
          return `	export let ${param} = "${value}";`;
        }).join("\n")}
<\/script>

`);
      }
      const {
        js: { code, map }
      } = compiler.compile(s.toString(), {
        generate: loadOptions?.ssr || ssr ? "ssr" : "dom",
        dev: process.env.NODE_ENV === "development",
        hydratable: true,
        css: false
      });
      return { code, map };
    }
  );
  return plugin(baseOptions);
};
const htmlEscapes = {
  "&": "&amp;",
  "<": "&lt;",
  ">": "&gt;",
  '"': "&quot;",
  "'": "&#39;",
  "{": "&#123;",
  "}": "&#125;"
};
function escapeHtml(html) {
  return html.replace(/[&<>"'{}]/g, (chr) => htmlEscapes[chr]);
}
function transformAndRetriveReplacements(code, regExp) {
  const replacements = [];
  const transformedCode = code.replace(regExp, (match, value, param, index) => {
    replacements.push({
      start: index,
      length: match.length,
      end: index + match.length,
      param: param || value.trim(),
      value
    });
    return value;
  });
  const params = replacements.filter(
    ({ param }, index, array) => array.findIndex((i) => i.param === param) === index
  );
  return { transformedCode, replacements, params };
}

exports.syntaxHighlight = syntaxHighlight;
module.exports = Object.assign(exports.default, exports);
//# sourceMappingURL=index.js.map
